@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009-2018 Codership Oy <info@codership.com>
+ * Copyright (C) 2009-2021 Codership Oy <info@codership.com>
  */
 
 #ifndef __GCACHE_H__
@@ -39,7 +39,9 @@ namespace gcache
          * Creates a new gcache file in "gcache.name" conf parameter or
          * in data_dir. If file already exists, it gets overwritten.
          */
-        GCache (gu::Config& cfg, const std::string& data_dir);
+        GCache (ProgressCallback*  pcb,
+                gu::Config&        cfg,
+                const std::string& data_dir);
 
         virtual ~GCache();
 
@@ -67,7 +69,7 @@ namespace gcache
          * Assign sequence number to buffer pointed to by ptr
          */
         void seqno_assign (const void* ptr,
-                           int64_t     seqno_g,
+                           seqno_t     seqno_g,
                            uint8_t     type,
                            bool        skip);
 
@@ -75,31 +77,31 @@ namespace gcache
          * Mark buffer to be skipped
          */
         void seqno_skip (const void* ptr,
-                         int64_t     seqno_g,
+                         seqno_t     seqno_g,
                          uint8_t     type);
 
         /*!
          * Release (free) buffers up to seqno
          */
-        void seqno_release (int64_t seqno);
+        void seqno_release (seqno_t seqno);
 
         /*!
          * Returns smallest seqno present in history
          */
-        int64_t seqno_min() const
+        seqno_t seqno_min() const
         {
             gu::Lock lock(mtx);
             if (gu_likely(!seqno2ptr.empty()))
-                return seqno2ptr.begin()->first;
+                return seqno2ptr.index_begin();
             else
-                return -1;
+                return SEQNO_ILL;
         }
 
         /*!
          * Move lock to a given seqno.
          * @throws gu::NotFound if seqno is not in the cache.
          */
-        void  seqno_lock (int64_t const seqno_g);
+        void  seqno_lock (seqno_t const seqno_g);
 
         /*!
          * Get pointer to buffer identified by seqno.
@@ -107,7 +109,7 @@ namespace gcache
          * The buffer will need to be "freed" again.
          * @throws NotFound
          */
-        const void* seqno_get_ptr (int64_t seqno_g, ssize_t& size);
+        const void* seqno_get_ptr (seqno_t seqno_g, ssize_t& size);
 
         class Buffer
         {
@@ -134,7 +136,7 @@ namespace gcache
                 return *this;
             }
 
-            int64_t           seqno_g() const { return seqno_g_; }
+            seqno_t           seqno_g() const { return seqno_g_; }
             const gu::byte_t* ptr()     const { return ptr_;     }
             ssize_type        size()    const { return size_;    }
             bool              skip()    const { return skip_;    }
@@ -147,7 +149,7 @@ namespace gcache
                 ptr_ = reinterpret_cast<const gu::byte_t*>(p);
             }
 
-            void set_other (int64_t g, ssize_type s, bool skp, uint8_t t)
+            void set_other (seqno_t g, ssize_type s, bool skp, uint8_t t)
             {
                 assert(s > 0);
                 seqno_g_ = g; size_ = s; skip_ = skp, type_ = t;
@@ -155,7 +157,7 @@ namespace gcache
 
         private:
 
-            int64_t           seqno_g_;
+            seqno_t           seqno_g_;
             const gu::byte_t* ptr_;
             ssize_type        size_;
             bool              skip_;
@@ -171,7 +173,7 @@ namespace gcache
          *
          * @retval number of buffers filled (<= v.size())
          */
-        size_t seqno_get_buffers (std::vector<Buffer>& v, int64_t start);
+        size_t seqno_get_buffers (std::vector<Buffer>& v, seqno_t start);
 
         /*!
          * Releases any seqno locks present.
@@ -230,7 +232,6 @@ namespace gcache
             params;
 
         gu::Mutex       mtx;
-        gu::Cond        cond;
 
         seqno2ptr_t     seqno2ptr;
         gu::UUID        gid;
@@ -243,9 +244,11 @@ namespace gcache
         long long       reallocs;
         long long       frees;
 
-        int64_t         seqno_locked;
-        int64_t         seqno_max;
-        int64_t         seqno_released;
+        seqno_t         seqno_max;
+        seqno_t         seqno_released;
+
+        seqno_t         seqno_locked;
+        int             seqno_locked_count;
 
 #ifndef NDEBUG
         std::set<const void*> buf_tracker;
@@ -254,10 +257,10 @@ namespace gcache
         void discard_buffer (BufferHeader* bh);
 
         /* returns true when successfully discards all seqnos up to s */
-        bool discard_seqno (int64_t s);
+        bool discard_seqno (seqno_t s);
 
         /* discards all seqnos greater than s */
-        void discard_tail (int64_t s);
+        void discard_tail (seqno_t s);
 
         // disable copying
         GCache (const GCache&);
