@@ -7,13 +7,15 @@
 #include "gcs_params.hpp"
 #include "gcs_fc.hpp" // gcs_fc_hard_limit_fix
 
-#define __STDC_FORMAT_MACROS
-#include <inttypes.h>
+#include "gu_inttypes.hpp"
+#include "gu_config.hpp" // gu::Config::Flag
+
 #include <cerrno>
 
 const char* const GCS_PARAMS_FC_FACTOR         = "gcs.fc_factor";
 const char* const GCS_PARAMS_FC_LIMIT          = "gcs.fc_limit";
 const char* const GCS_PARAMS_FC_MASTER_SLAVE   = "gcs.fc_master_slave";
+const char* const GCS_PARAMS_FC_SINGLE_PRIMARY = "gcs.fc_single_primary";
 const char* const GCS_PARAMS_FC_DEBUG          = "gcs.fc_debug";
 const char* const GCS_PARAMS_SYNC_DONOR        = "gcs.sync_donor";
 const char* const GCS_PARAMS_MAX_PKT_SIZE      = "gcs.max_packet_size";
@@ -27,6 +29,7 @@ const char* const GCS_PARAMS_SM_DUMP           = "gcs.sm_dump";
 static const char* const GCS_PARAMS_FC_FACTOR_DEFAULT         = "1.0";
 static const char* const GCS_PARAMS_FC_LIMIT_DEFAULT          = "16";
 static const char* const GCS_PARAMS_FC_MASTER_SLAVE_DEFAULT   = "no";
+static const char* const GCS_PARAMS_FC_SINGLE_PRIMARY_DEFAULT = "no";
 static const char* const GCS_PARAMS_FC_DEBUG_DEFAULT          = "0";
 static const char* const GCS_PARAMS_SYNC_DONOR_DEFAULT        = "no";
 static const char* const GCS_PARAMS_MAX_PKT_SIZE_DEFAULT      = "64500";
@@ -40,29 +43,43 @@ gcs_params_register(gu_config_t* conf)
     bool ret = 0;
 
     ret |= gu_config_add (conf, GCS_PARAMS_FC_FACTOR,
-                          GCS_PARAMS_FC_FACTOR_DEFAULT);
+                          GCS_PARAMS_FC_FACTOR_DEFAULT,
+                          gu::Config::Flag::type_double);
     ret |= gu_config_add (conf, GCS_PARAMS_FC_LIMIT,
-                          GCS_PARAMS_FC_LIMIT_DEFAULT);
+                          GCS_PARAMS_FC_LIMIT_DEFAULT,
+                          gu::Config::Flag::type_integer);
     ret |= gu_config_add (conf, GCS_PARAMS_FC_MASTER_SLAVE,
-                          GCS_PARAMS_FC_MASTER_SLAVE_DEFAULT);
+                          GCS_PARAMS_FC_MASTER_SLAVE_DEFAULT,
+                          gu::Config::Flag::deprecated |
+                          gu::Config::Flag::type_bool);
+    ret |= gu_config_add (conf, GCS_PARAMS_FC_SINGLE_PRIMARY,
+                          GCS_PARAMS_FC_SINGLE_PRIMARY_DEFAULT,
+                          gu::Config::Flag::read_only |
+                          gu::Config::Flag::type_bool);
     ret |= gu_config_add (conf, GCS_PARAMS_FC_DEBUG,
-                          GCS_PARAMS_FC_DEBUG_DEFAULT);
+                          GCS_PARAMS_FC_DEBUG_DEFAULT,
+                          gu::Config::Flag::type_integer);
     ret |= gu_config_add (conf, GCS_PARAMS_SYNC_DONOR,
-                          GCS_PARAMS_SYNC_DONOR_DEFAULT);
+                          GCS_PARAMS_SYNC_DONOR_DEFAULT,
+                          gu::Config::Flag::type_bool);
     ret |= gu_config_add (conf, GCS_PARAMS_MAX_PKT_SIZE,
-                          GCS_PARAMS_MAX_PKT_SIZE_DEFAULT);
+                          GCS_PARAMS_MAX_PKT_SIZE_DEFAULT,
+                          gu::Config::Flag::type_integer);
 
     char tmp[32] = { 0, };
     snprintf (tmp, sizeof(tmp) - 1, "%lld",
               (long long)GCS_PARAMS_RECV_Q_HARD_LIMIT_DEFAULT);
-    ret |= gu_config_add (conf, GCS_PARAMS_RECV_Q_HARD_LIMIT, tmp);
+    ret |= gu_config_add (conf, GCS_PARAMS_RECV_Q_HARD_LIMIT, tmp,
+                          gu::Config::Flag::type_integer);
 
     ret |= gu_config_add (conf, GCS_PARAMS_RECV_Q_SOFT_LIMIT,
-                          GCS_PARAMS_RECV_Q_SOFT_LIMIT_DEFAULT);
+                          GCS_PARAMS_RECV_Q_SOFT_LIMIT_DEFAULT,
+                          gu::Config::Flag::type_double);
     ret |= gu_config_add (conf, GCS_PARAMS_MAX_THROTTLE,
-                          GCS_PARAMS_MAX_THROTTLE_DEFAULT);
+                          GCS_PARAMS_MAX_THROTTLE_DEFAULT,
+                          gu::Config::Flag::type_double);
 #ifdef GCS_SM_DEBUG
-    ret |= gu_config_add (conf, GCS_PARAMS_SM_DUMP, "0");
+    ret |= gu_config_add (conf, GCS_PARAMS_SM_DUMP, "0", 0);
 #endif /* GCS_SM_DEBUG */
     return ret;
 }
@@ -178,6 +195,18 @@ params_init_double (gu_config_t* conf, const char* const name,
     return 0;
 }
 
+static void deprecation_warning(gu_config_t* config,
+                                const char* deprecated,
+                                const char* current)
+{
+    if (gu_config_is_set(config, deprecated))
+    {
+        gu_warn("Option '%s' is deprecated and will be removed in the "
+                "future versions, please use '%s' instead. ",
+                deprecated, current);
+    }
+}
+
 long
 gcs_params_init (struct gcs_params* params, gu_config_t* config)
 {
@@ -210,7 +239,19 @@ gcs_params_init (struct gcs_params* params, gu_config_t* config)
     // allow for some meta overhead
 
     if ((ret = params_init_bool (config, GCS_PARAMS_FC_MASTER_SLAVE,
-                                 &params->fc_master_slave))) return ret;
+                                 &params->fc_single_primary))) return ret;
+    if (params->fc_single_primary)
+    {
+        // if GCS_PARAMS_FC_MASTER_SLAVE was used, log deprecation warning
+        deprecation_warning(config, GCS_PARAMS_FC_MASTER_SLAVE,
+                            GCS_PARAMS_FC_SINGLE_PRIMARY);
+    }
+    else
+    {
+        // Overrides deprecated GCS_PARAMS_FC_MASTER_SLAVE if set
+        if ((ret = params_init_bool (config, GCS_PARAMS_FC_SINGLE_PRIMARY,
+                                     &params->fc_single_primary))) return ret;
+    }
 
     if ((ret = params_init_bool (config, GCS_PARAMS_SYNC_DONOR,
                                  &params->sync_donor))) return ret;
